
/*****************************************************************
MODULE:		pbitprocs.c
PROGRAMMERS:    John White and Yaoen Zhang
DESCRIPTION:    This module contains callbacks, event handlers,
		and support functions for pbitdraw.c
*****************************************************************/

#include "p2.h"
extern int current_map;
extern void put_message();

/*****************************************************************
FUNCTION:	pixel_set()
PURPOSE:        sets bits in the viewing box corresponding to rectangles 
           	in the tiling box	
PARAMETERS:     display - display i.d. of widget to be drawn upon
	        window  - window i.d. of widget to be drawn upon
		context - corresponding graphics context
                x,y     - coordinates of pixel to set	
CALLED BY:      render_grid(), select_square(), and clear_tile()
***************************************************/
void pixel_set(display, window, context, x, y)
Display *display;
Window  window; 
GC      context;	
Position x,y;
{
int i,j;

for(i=0;i<UNIT_AREA-1;i++)
  for(j=0;j<UNIT_AREA-1;j++)
	XDrawPoint(display, window, context, 
		(x*(UNIT_AREA-1)) + i + X_VIEW_OFFSET ,
		(y*(UNIT_AREA-1)) + j  + Y_VIEW_OFFSET);
}
/********************************************************************
 FUNCTION:	rectangle_set: 
 PURPOSE:	sets user-selected rectangles in the tiling box
 PARAMETERS:    display - display i.d. of widget to be drawn upon
                window  - window i.d of widget to be drawn upon
	        context - graphics context of corresponding widget
		x,y     - index values referencing a given rectangle
CALLED BY:      render_grid(), select_square(), and clear_tile()
******************************************************************/
void rectangle_set(display, window, context, x, y)
Display *display;
Window  window; 
GC      context;	
Position x,y;
{
        XFillRectangle(display, window, context, x*SQUARE_SIZE +
	X_TILE_OFFSET + RIM_WIDTH, y*SQUARE_SIZE + Y_TILE_OFFSET + 
	RIM_WIDTH, SQUARE_SIZE - 2*RIM_WIDTH, SQUARE_SIZE - 2*RIM_WIDTH);
}

/******************************************************************
FUNCTION:	render_grid
PURPOSE:        Creates rectangular grid on tiling_box
		and fills any squares that are specificed in the
		are specified in the Tile_type structure.
		Set points are also displayed as
		pixels in the viewing_box
PARAMETERS:     event_win - widget where the event is registered
		data      - Pointer to Info structure containing
			    important widget data
		event	  - Pointer to the event which triggered
			    this function
CALLED BY:	This is an event handler triggered once for each
		time the pbitdraw application is called.
*******************************************************************/
void render_grid(event_win,data,event)
Widget event_win;
Info *data;
XEvent *event;
{
Position x,y;

XSetForeground(data->w1_display,data->gc1, 1);
XSetForeground(data->w1_display,data->gc2, 1);
/* creates rectangular grid */

for(x=X_TILE_OFFSET;x<=X_TILE_MAX;x+=SQUARE_SIZE)
	for(y=Y_TILE_OFFSET;y<=Y_TILE_MAX;y+=SQUARE_SIZE)
		XDrawRectangle(data->w1_display,data->w1_window,
		         data->gc1,x,y,SQUARE_SIZE,SQUARE_SIZE);

/* Sets any rectangles and pixels SET in tile */
for(x=0;x<TILE_DIM;x++)
	for(y=0;y<TILE_DIM;y++)
		if(tile[current_map][x][y] == SET)
		{
		rectangle_set(data->w1_display, data->w1_window,
	                      data->gc1, x, y);
		pixel_set(data->w2_display, data->w2_window,
			      data->gc2, x, y);
		}		
}
/******************************************************************
FUNCTION:       copy_tiling	
PURPOSE:        Copies the values of a source Tile_type to a
		destination Tile_type.  It is used to copy
		between the local and global data structures
		representing the texture map
PARAMETERS:     source_tile      -   Tile_type variable 
		destination_tile -   Tile_type variable
CALLED BY:      init_info and save_image	
*******************************************************************/
copy_tiling (source_tile, destination_tile)
Tile_type source_tile, destination_tile;
{
int i,j;

for(i=0;i<TILE_DIM;i++)
	for(j=0;j<TILE_DIM;j++)
		destination_tile[i][j] = source_tile[i][j];
}
/******************************************************************
FUNCTION:       to_x_coord 
PURPOSE:        Converts an x value representing cursor position
		in the tiling box to a corresponding x index
		value of the rectangle in which the cursor is
		located
PARAMETERS:     x  -  Position variable 
CALLED BY:      select_square 
*************************************************************/
Position to_x_coord(x)
Position x;
{
Position i;

return (i = (x - X_TILE_OFFSET) / SQUARE_SIZE);
}

/******************************************************************
FUNCTION:       to_y_coord 
PURPOSE:        Converts an y value representing cursor position
		in the tiling box to a corresponding y index
		value of the rectangle in which the cursor is
		located
PARAMETERS:     y  -  Position variable 
CALLED BY:      select_square 
*****************************************************************/
Position to_y_coord(y)
Position y;
{
Position j;

return (j = (y - Y_TILE_OFFSET) / SQUARE_SIZE);
}

/******************************************************************
FUNCTION:       start_select_square 
PURPOSE:        Highlight a user selected rectangle and its
		corresponding pixel in the viewing box and
		SET or UNSET the tile data structure. Setting
		is through exlcusive or (xor). 
PARAMETERS:     event_win    -    window in which the event occurred 
		data	     -    pointer to info structure containing
					useful widget information
		event	     -    pointer to XEvent which trigged this
					function
CALLED BY:      This is an event handler for ButtonPress in the
		tiling box. 
*****************************************************************/
void start_select_square(event_win, data, event)
Widget event_win;
Info *data;
XEvent *event;
{
Position x,y;

x = to_x_coord(event->xbutton.x);
y = to_y_coord(event->xbutton.y);
if (x >= 0 && x < TILE_DIM && y >= 0 && y < TILE_DIM)
    { 

      data->tmp_tile[x][y] = !(data->tmp_tile[x][y]);
      if (data->tmp_tile[x][y] == SET)
       {
          data->color_flag = SET;
	  XSetForeground(data->w1_display,data->gc1, 1);
	  XSetForeground(data->w1_display,data->gc2, 1);
       }
      else
       { 
          data->color_flag = NOT_SET;
	  XSetForeground(data->w1_display,data->gc1, 0);
	  XSetForeground(data->w1_display,data->gc2, 0);
       }
		
      rectangle_set(data->w1_display, data->w1_window, data->gc1, x, y);
      pixel_set(data->w2_display, data->w2_window, data->gc2, x, y);
    }
}

/******************************************************************
FUNCTION:       move_select_square 
PURPOSE:        Highlight a user selected rectangle and its
		corresponding pixel in the viewing box and
		SET or UNSET the tile data structure. Setting
		is through exlcusive or (xor). 
PARAMETERS:     event_win    -    window in which the event occurred 
		data	     -    pointer to info structure containing
					useful widget information
		event	     -    pointer to XEvent which trigged this
					function
CALLED BY:      This is an event handler for ButtonPress in the
		tiling box. 
*****************************************************************/
void move_select_square(event_win, data, event)
Widget event_win;
Info *data;
XEvent *event;
{
Position x,y;

if(data->color_flag != NOWRITE)
{
    x = to_x_coord(event->xbutton.x);
    y = to_y_coord(event->xbutton.y);
    if (x >= 0 && x < TILE_DIM && y >= 0 && y < TILE_DIM)
      { 
     	 rectangle_set(data->w1_display, data->w1_window, data->gc1, x, y);

         pixel_set(data->w2_display, data->w2_window, data->gc2, x, y);

         data->tmp_tile[x][y] = data->color_flag;
       
      }
 }
}

/******************************************************************
FUNCTION:       stop_select_square 
PURPOSE:        Highlight a user selected rectangle and its
		corresponding pixel in the viewing box and
		SET or UNSET the tile data structure. Setting
		is through exlcusive or (xor). 
PARAMETERS:     event_win    -    window in which the event occurred 
		data	     -    pointer to info structure containing
					useful widget information
		event	     -    pointer to XEvent which trigged this
					function
CALLED BY:      This is an event handler for ButtonPress in the
		tiling box. 
*****************************************************************/
void stop_select_square(event_win, data, event)
Widget event_win;
Info *data;
XEvent *event;
{

data->color_flag = NOWRITE;
XSetForeground(data->w1_display,data->gc1, 1);
XSetForeground(data->w1_display,data->gc2, 1);
}

/******************************************************************
FUNCTION:       destroypopup 
PURPOSE:        Terminate execution of the popup shell widget which 
		contains the create_tiling function.
PARAMETERS:     widget	          - widget with popup parent 
		closure, callData - unused, required by callback
CALLED BY:      This is a callback from the quit_command in 
		create_tiling(). 
******************************************************************/
void
destroypopup(widget, data , callData)
Widget widget;
canvas_data *data;
XtPointer callData;
{
	Widget form=XtParent(XtParent(XtParent(widget)));
        XtDestroyWidget(form);
	put_message(data,"");	/* clear message window */
}
 
/******************************************************************
FUNCTION:       clear_tile 
PURPOSE:	This clears the tiling_box, the viewing_box,
		and the local Tile_type structure holding a
		texture map.
PARAMETERS:     event_win  - unused, required by event handler 
		data	   - pointer to Info structure containing
				important widget information 
		event      - unused, required by event handler
CALLED BY:      This is an event handler for the clear_command in 
		create_tiling(). 
*****************************************************************/

void
clear_tile (event_win, data, event)
Widget event_win;
Info *data;
XEvent *event;
{
Position x,y;

XSetForeground(data->w1_display,data->gc1, 0);
XSetForeground(data->w1_display,data->gc2, 0);

for(x=0;x<TILE_DIM;x++)
	for(y=0;y<TILE_DIM;y++)
		if(data->tmp_tile[x][y] == SET)
		{

		rectangle_set(data->w1_display, data->w1_window,
	                      data->gc1, x, y);
		pixel_set(data->w2_display, data->w2_window,
			      data->gc2, x, y);
                data->tmp_tile[x][y] = NOT_SET;

		}		

XDrawString(data->w1_display,data->w1_window,data->gc1,40,9,"Image not yet written",
		strlen("Image not yet written"));
XSetForeground(data->w1_display,data->gc1, 1);
XSetForeground(data->w1_display,data->gc2, 1);
}

/******************************************************************
FUNCTION:       rev_tile 
PURPOSE:	This reverses the current tiling
PARAMETERS:     event_win  - unused, required by event handler 
		data	   - pointer to Info structure containing
				important widget information 
		event      - unused, required by event handler
CALLED BY:      This is an event handler for the clear_command in 
		create_tiling(). 
*****************************************************************/
void
rev_tile (event_win, data, event)
Widget event_win;
Info *data;
XEvent *event;
{
Position x,y;

for(x=0;x<TILE_DIM;x++)
	for(y=0;y<TILE_DIM;y++)  {
		if(data->tmp_tile[x][y] == SET)
		   {
                	data->tmp_tile[x][y] = NOT_SET;
			XSetForeground(data->w1_display,data->gc1, 0);
			XSetForeground(data->w1_display,data->gc2, 0);
                    }
	        else	
		    {
                	data->tmp_tile[x][y] = SET;
			XSetForeground(data->w1_display,data->gc1, 1);
			XSetForeground(data->w1_display,data->gc2, 1);
		     }

		rectangle_set(data->w1_display, data->w1_window,
	                      data->gc1, x, y);
		pixel_set(data->w2_display, data->w2_window,
			      data->gc2, x, y);

		}		
XSetForeground(data->w1_display,data->gc1, 0);
XDrawString(data->w1_display,data->w1_window,data->gc1,40,9,"Image not yet written",
		strlen("Image not yet written"));
XSetForeground(data->w1_display,data->gc1, 1);
}
/******************************************************************
FUNCTION:       save_image 
PURPOSE:	This saves the texture map stored in the local 
		Tile_type variable into the global tile structure.
PARAMETERS:     event_win  - unused, required by event handler 
		data	   - pointer to Info structure containing
				important widget information 
		event      - unused, required by event handler
*****************************************************************/
void
save_image (event_win, data, event)
Widget event_win;
Info *data;
XEvent *event;
{
void draw_main_tile();

data->write = SET;
copy_tiling(data->tmp_tile, tile[current_map]);

XSetForeground(data->w1_display,data->gc1, 0);
XDrawString(data->w1_display,data->w1_window,data->gc1,40,9,"Image not yet written",
		strlen("Image not yet written"));
XSetForeground(data->w1_display,data->gc1, 1);

draw_main_tile(data);
}

/******************************************************************
FUNCTION:       next_image 
PURPOSE:	This loads the next texture to be created or modified. 
		It will circle through the six possible maps.
PARAMETERS:     event_win  - unused, required by event handler 
		data	   - pointer to Info structure containing
				important widget information 
		event      - unused, required by event handler
*****************************************************************/
 void
     next_image (event_win, data, event)
     Widget event_win;
     Info *data;
     XEvent *event;
     {
     void bitputmsg();
     int x,y;
     char mstr[MAXSTRING];
     static char str1[] = "Image not yet written";
   
     if(data->write == NOT_SET)
         {
	     XSetForeground(data->w1_display,data->gc1, 132);
             XDrawString(data->w1_display,data->w1_window,data->gc1,40,9,str1
,
                             strlen(str1));
         }
     else
          {
	     XSetForeground(data->w1_display,data->gc1, 0);
             XDrawString(data->w1_display,data->w1_window,data->gc1,40,9,str1
,
                        strlen(str1));
             clear_tile (event_win, data, event);
             current_map = (current_map + 1) % MAX_TEXTURE_MAPS;
             /* put a message telling the current tiling */
             if(current_map == 4)
             sprintf(mstr,"Tiling the No. %d texture map", current_map + 2);
                     else
             sprintf(mstr,"Tiling the No. %d texture map",(current_map + 2)%MAX_TEXTURE_MAPS);

             bitputmsg(data,mstr);
             copy_tiling(tile[current_map], data->tmp_tile);
   
            for(x=0;x<TILE_DIM;x++)
                 for(y=0;y<TILE_DIM;y++)
                     if(tile[current_map][x][y] == SET)
                     {
                     rectangle_set(data->w1_display, data->w1_window,
                                   data->gc1, x, y);
                     pixel_set(data->w2_display, data->w2_window,
                                   data->gc2, x, y);
                     }
          }
	     XSetForeground(data->w1_display,data->gc1, 1);
	     data->write = !data->write;

}

/******************************************************************
FUNCTION:       prev_image 
PURPOSE:	This loads the previous texture to be created or modified. 
		It will circle through the six possible maps.
PARAMETERS:     event_win  - unused, required by event handler 
		data	   - pointer to Info structure containing
				important widget information 
		event      - unused, required by event handler
*****************************************************************/

 void
     prev_image (event_win, data, event)
     Widget event_win;
     Info *data;
     XEvent *event;
     {
     void bitputmsg();
     int x,y;
     char mstr[MAXSTRING];
     static char str1[] = "Image not yet written";
   
     if(data->write == NOT_SET)
         {
	     XSetForeground(data->w1_display,data->gc1, 132);
             XDrawString(data->w1_display,data->w1_window,data->gc1,40,9,str1
,
                             strlen(str1));
         }
     else
          {
	     XSetForeground(data->w1_display,data->gc1, 0);
             XDrawString(data->w1_display,data->w1_window,data->gc1,40,9,str1
,
                        strlen(str1));
             clear_tile (event_win, data, event);
             current_map = (current_map - 1) % MAX_TEXTURE_MAPS;
             /* put a message telling the current tiling */
             if(current_map == 4)
             sprintf(mstr,"Tiling the No. %d texture map", current_map + 2);
                     else
             sprintf(mstr,"Tiling the No. %d texture map",(current_map + 2)%MAX_TEXTURE_MAPS);

             bitputmsg(data,mstr);
             copy_tiling(tile[current_map], data->tmp_tile);
   
            for(x=0;x<TILE_DIM;x++)
                 for(y=0;y<TILE_DIM;y++)
                     if(tile[current_map][x][y] == SET)
                     {
                     rectangle_set(data->w1_display, data->w1_window,
                                   data->gc1, x, y);
                     pixel_set(data->w2_display, data->w2_window,
                                   data->gc2, x, y);
                     }
          }
	     XSetForeground(data->w1_display,data->gc1, 1);
	     data->write = !data->write;

}
/********************************************************************
FUCNTION:       bitputmsg
PURPOSE:        Writes a message in the message window
PARAMETERS:     data    - pointer to Info structure containing
                        important widget information
                message - pointer to a string
*******************************************************************/
void bitputmsg(data,message)
  Info *data;
  char *message;
{
char padding[MAXSTRING], str[MAXSTRING];
int i;
Arg     args[1];

sprintf(str,message);
        XtSetArg(args[0],XtNlabel,str);
        XtSetValues(data->help,args,ONE);
}

/********************************************************************
FUCNTION:       draw_main_tile 
PURPOSE:        Draws tile patterns on the main window 
PARAMETERS:     data    - pointer to Info structure containing
                        important widget information
*******************************************************************/
void draw_main_tile(data)
Info *data;
{
 int j,k,l,m,temp;
   
       temp=(current_map+1)%6;
       for(j=0;j<TILE_DIM;j++)
         for(k=0;k<TILE_DIM;k++)
            if(tile[current_map][j][k] == SET)
	    {
              for(l=0;l<UNIT_AREA-1;l++)
                 for(m=0;m<UNIT_AREA-1;m++)
		  {
   
		     XSetForeground(XtDisplay(data->tileview),data->gc1,1);
                     XDrawPoint(XtDisplay(data->tileview),XtWindow(data->tileview),
                       data->gc1, ((temp%3)*40+8) + (j*(UNIT_AREA-1)) + l,
                                 ((temp/3)*40+8) + (k*(UNIT_AREA-1)) + m);
		  }
             } 
           else 
		{
                 for(l=0;l<UNIT_AREA-1;l++)
                     for(m=0;m<UNIT_AREA-1;m++)
			{
	    		 XSetForeground(XtDisplay(data->tileview),data->gc1,0);
             		 XDrawPoint(XtDisplay(data->tileview),XtWindow(data->tileview),
             			data->gc1, ((temp%3)*40+8) + (j*(UNIT_AREA-1)) + l,
             		((temp/3)*40+8) + (k*(UNIT_AREA-1)) + m);
			}
		}
       XSetForeground(XtDisplay(data->tileview),data->gc1,1);

       if(current_map == 5)
            XDrawRectangle(XtDisplay(data->tileview),XtWindow(data->tileview),data->gc1,
       		8, 8, 32, 32);
	else	
       		XDrawRectangle(XtDisplay(data->tileview),XtWindow(data->tileview),data->gc1,
       			(((current_map+1)%3)*40+8), (((current_map+1)/3)*40+8), 32, 32);

      }
